package repo

import (
	"fmt"
	"net/http"
	"net/url"
	"strings"
	"time"

	"code.gitea.io/gitea/models"
	"code.gitea.io/gitea/modules/context"
	"code.gitea.io/gitea/modules/log"
	"github.com/360EntSecGroup-Skylar/excelize/v2"
)

type DateCloudbrainNum struct {
	Date                      string         `json:"date"`
	CloudOneJobTypeRes        map[string]int `json:"cloudOneJobTypeRes"`
	CloudTwoJobTypeRes        map[string]int `json:"cloudTwoJobTypeRes"`
	IntelligentNetJobTypeRes  map[string]int `json:"intelligentNetJobTypeRes"`
	CloudBrainPeriodNum       map[int]int    `json:"cloudBrainPeriodNum"`
	CloudBrainComputeResource map[string]int `json:"cloudBrainComputeResource"`
}
type DateCloudbrainInfo struct {
	Date                      string         `json:"date"`
	CloudBrainPeriodNum       map[int]int    `json:"cloudBrainPeriodNum"`
	CloudBrainComputeResource map[string]int `json:"cloudBrainComputeResource"`
}
type CloudbrainsStatusAnalysis struct {
	JobWaitingCount   int64 `json:"jobWaitingCount"`
	JobRunningCount   int64 `json:"jobRunningCount"`
	JobStoppedCount   int64 `json:"jobStoppedCount"`
	JobCompletedCount int64 `json:"jobCompletedCount"`
	JobFailedCount    int64 `json:"jobFailedCount"`
	JobKilledCount    int64 `json:"jobKilledCount"`
	JobInitCount      int64 `json:"jobInitCount"`
}

func GetAllCloudbrainsOverview(ctx *context.Context) {
	recordCloudbrain, err := models.GetRecordBeginTime()
	if err != nil {
		log.Error("Can not get recordCloudbrain", err)
		ctx.Error(http.StatusBadRequest, ctx.Tr("repo.record_begintime_get_err"))
		return
	}
	recordBeginTime := recordCloudbrain[0].Cloudbrain.CreatedUnix
	now := time.Now()
	beginTime := time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location())
	endTime := now
	todayCreatorCount, err := models.GetTodayCreatorCount(beginTime, endTime)
	if err != nil {
		log.Error("Can not query todayCreatorCount.", err)
		return
	}
	creatorCount, err := models.GetCreatorCount()
	if err != nil {
		log.Error("Can not query creatorCount.", err)
		return
	}
	todayStatusResult := make(map[string]int)
	cloudBrainNum := make(map[int]int)
	cloudBrainOneDuration := int64(0)
	cloudBrainTwoDuration := int64(0)
	intelligentNetDuration := int64(0)
	todayNewJobCount := 0
	page := 1
	pagesize := 1000
	count := pagesize
	for count == pagesize && count != 0 {
		cloudbrains, _, err := models.CloudbrainAllStatic(&models.CloudbrainsOptions{
			ListOptions: models.ListOptions{
				Page:     page,
				PageSize: pagesize,
			},
			Type:          models.TypeCloudBrainAll,
			BeginTimeUnix: int64(recordBeginTime),
			EndTimeUnix:   endTime.Unix(),
		})
		if err != nil {
			ctx.ServerError("Get cloudbrains failed:", err)
			return
		}

		for _, cloudbrain := range cloudbrains {
			if int64(cloudbrain.Cloudbrain.CreatedUnix) >= beginTime.Unix() && int64(cloudbrain.Cloudbrain.CreatedUnix) < endTime.Unix() {
				todayNewJobCount += 1
				if _, ok := todayStatusResult[cloudbrain.Status]; !ok {
					todayStatusResult[cloudbrain.Status] = 1
				} else {
					todayStatusResult[cloudbrain.Status] += 1
				}
			}

			if _, ok := cloudBrainNum[cloudbrain.Cloudbrain.Type]; !ok {
				cloudBrainNum[cloudbrain.Cloudbrain.Type] = 1
			} else {
				cloudBrainNum[cloudbrain.Cloudbrain.Type] += 1
			}

			if cloudbrain.Cloudbrain.Type == models.TypeCloudBrainOne {
				cloudBrainOneDuration = cloudBrainOneDuration + cloudbrain.Cloudbrain.Duration
			}
			if cloudbrain.Cloudbrain.Type == models.TypeCloudBrainTwo {
				cloudBrainTwoDuration = cloudBrainTwoDuration + cloudbrain.Cloudbrain.Duration
			}
			if cloudbrain.Cloudbrain.Type == models.TypeC2Net {
				intelligentNetDuration = intelligentNetDuration + cloudbrain.Cloudbrain.Duration
			}

		}
		count = len(cloudbrains)
		page += 1
	}

	statusNameList := []string{string(models.ModelArtsTrainJobCompleted), string(models.JobFailed), string(models.ModelArtsTrainJobInit),
		string(models.JobRunning), string(models.ModelArtsStartFailed), string(models.JobStopped), string(models.JobSucceeded),
		string(models.JobWaiting), string(models.ModelArtsTrainJobKilled)}
	for _, v := range statusNameList {
		if _, ok := todayStatusResult[v]; !ok {
			todayStatusResult[v] = 0
		}
	}

	cloudBrainTypeList := []int{0, 1, 2}
	for _, v := range cloudBrainTypeList {
		if _, ok := cloudBrainNum[v]; !ok {
			cloudBrainNum[v] = 0
		}
	}

	todayRunningCount := todayStatusResult[string(models.JobRunning)]
	todayCompletedCount := todayStatusResult[string(models.ModelArtsTrainJobCompleted)] + todayStatusResult[string(models.JobFailed)] +
		todayStatusResult[string(models.ModelArtsStartFailed)] + todayStatusResult[string(models.JobStopped)] + todayStatusResult[string(models.JobSucceeded)] + todayStatusResult[string(models.ModelArtsTrainJobKilled)]
	todayWaitingCount := todayStatusResult[string(models.ModelArtsTrainJobInit)] + todayStatusResult[string(models.JobWaiting)]

	ctx.JSON(http.StatusOK, map[string]interface{}{
		"recordBeginTime":        recordBeginTime,
		"updateTime":             now.Unix(),
		"cloudBrainNum":          cloudBrainNum,
		"cloudBrainOneDuration":  cloudBrainOneDuration,
		"cloudBrainTwoDuration":  cloudBrainTwoDuration,
		"intelligentNetDuration": intelligentNetDuration,
		"todayCreatorCount":      todayCreatorCount,
		"creatorCount":           creatorCount,
		"todayRunningCount":      todayRunningCount,
		"todayCompletedCount":    todayCompletedCount,
		"todayWaitingCount":      todayWaitingCount,
		"todayNewJobCount":       todayNewJobCount,
	})
}

func GetAllCloudbrainsTrend(ctx *context.Context) {

	queryType := ctx.QueryTrim("type")
	now := time.Now()

	beginTimeStr := ctx.QueryTrim("beginTime")
	endTimeStr := ctx.QueryTrim("endTime")
	var beginTime time.Time
	var endTime time.Time
	dateCloudbrainNum := make([]DateCloudbrainNum, 0)
	var err error
	if queryType != "" {
		if queryType == "all" {
			recordCloudbrain, err := models.GetRecordBeginTime()
			if err != nil {
				log.Error("Can not get recordCloudbrain", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("repo.record_begintime_get_err"))
				return
			}
			brainRecordBeginTime := recordCloudbrain[0].Cloudbrain.CreatedUnix.AsTime()
			beginTime = brainRecordBeginTime
			endTime = now
			dateCloudbrainNum, err = getMonthCloudbrainNum(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getMonthCloudbrainNum.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getMonthCloudbrainNum_get_error"))
				return
			}
		} else if queryType == "today" {
			beginTime = time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location())
			// endTimeTemp = now
			endTime = beginTime.AddDate(0, 0, 1)
			dateCloudbrainNum, err = getDayCloudbrainNum(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getDayCloudbrainNum.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getDayCloudbrainNum_get_error"))
				return
			}
		} else if queryType == "yesterday" {
			beginTime = now.AddDate(0, 0, -1)
			beginTime = time.Date(beginTime.Year(), beginTime.Month(), beginTime.Day(), 0, 0, 0, 0, now.Location())
			endTime = time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location())
			dateCloudbrainNum, err = getDayCloudbrainNum(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getDayCloudbrainNum.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getDayCloudbrainNum_get_error"))
				return
			}
		} else if queryType == "last_7day" {
			beginTime = now.AddDate(0, 0, -6)
			beginTime = time.Date(beginTime.Year(), beginTime.Month(), beginTime.Day(), 0, 0, 0, 0, now.Location())
			endTime = now
			dateCloudbrainNum, err = getDayCloudbrainNum(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getDayCloudbrainNum.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getDayCloudbrainNum_get_error"))
				return
			}
		} else if queryType == "last_30day" {
			beginTime = now.AddDate(0, 0, -29)
			beginTime = time.Date(beginTime.Year(), beginTime.Month(), beginTime.Day(), 0, 0, 0, 0, now.Location())
			endTime = now
			dateCloudbrainNum, err = getDayCloudbrainNum(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getDayCloudbrainNum.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getDayCloudbrainNum_get_error"))
				return
			}
		} else if queryType == "current_month" {
			endTime = now
			beginTime = time.Date(endTime.Year(), endTime.Month(), 1, 0, 0, 0, 0, now.Location())
			dateCloudbrainNum, err = getDayCloudbrainNum(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getDayCloudbrainNum.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getDayCloudbrainNum_get_error"))
				return
			}
		} else if queryType == "current_year" {
			endTime = now
			beginTime = time.Date(endTime.Year(), 1, 1, 0, 0, 0, 0, now.Location())
			// endTimeTemp = beginTime.AddDate(0, 1, 0)
			dateCloudbrainNum, err = getMonthCloudbrainNum(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getMonthCloudbrainNum.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getMonthCloudbrainNum_get_error"))
				return
			}
		} else if queryType == "last_month" {

			lastMonthTime := now.AddDate(0, -1, 0)
			beginTime = time.Date(lastMonthTime.Year(), lastMonthTime.Month(), 1, 0, 0, 0, 0, now.Location())
			endTime = time.Date(now.Year(), now.Month(), 1, 0, 0, 0, 0, now.Location())
			dateCloudbrainNum, err = getDayCloudbrainNum(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getDayCloudbrainNum.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getDayCloudbrainNum_get_error"))
				return
			}
		}

	} else {
		if beginTimeStr == "" || endTimeStr == "" {
			//如果查询类型和开始时间结束时间都未设置，按queryType=all处理
			recordCloudbrain, err := models.GetRecordBeginTime()
			if err != nil {
				log.Error("Can not get recordCloudbrain", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("repo.record_begintime_get_err"))
				return
			}
			brainRecordBeginTime := recordCloudbrain[0].Cloudbrain.CreatedUnix.AsTime()
			beginTime = brainRecordBeginTime
			endTime = now
			dateCloudbrainNum, err = getMonthCloudbrainNum(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getMonthCloudbrainNum.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getMonthCloudbrainNum_get_error"))
				return
			}
		} else {
			beginTime, err = time.ParseInLocation("2006-01-02", beginTimeStr, time.Local)
			if err != nil {
				log.Error("Can not ParseInLocation.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("ParseInLocation_get_error"))
				return
			}
			endTime, err = time.ParseInLocation("2006-01-02", endTimeStr, time.Local)
			if err != nil {
				log.Error("Can not ParseInLocation.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("ParseInLocation_get_error"))
				return
			}
			if endTime.After(time.Now()) {
				endTime = time.Now()
			}
			dateCloudbrainNum, err = getDayCloudbrainNum(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getDayCloudbrainNum.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getDayCloudbrainNum_get_error"))
				return
			}
		}

	}
	ctx.JSON(http.StatusOK, map[string]interface{}{
		"timeCloudbrainNum": dateCloudbrainNum,
	})

}
func GetAllCloudbrainsTrendDetail(ctx *context.Context) {
	queryType := ctx.QueryTrim("type")
	now := time.Now()

	beginTimeStr := ctx.QueryTrim("beginTime")
	endTimeStr := ctx.QueryTrim("endTime")
	var beginTime time.Time
	var endTime time.Time
	var endTimeTemp time.Time
	dayCloudbrainInfo := make([]DateCloudbrainInfo, 0)
	var err error
	var count int
	if queryType != "" {
		if queryType == "all" {
			recordCloudbrain, err := models.GetRecordBeginTime()
			if err != nil {
				log.Error("Can not get recordCloudbrain", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("repo.record_begintime_get_err"))
				return
			}
			brainRecordBeginTime := recordCloudbrain[0].Cloudbrain.CreatedUnix.AsTime()
			beginTime = brainRecordBeginTime
			endTime = now
			dayCloudbrainInfo, count, err = getMonthCloudbrainInfo(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getDayCloudbrainInfo.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getDayCloudbrainInfo_get_error"))
				return
			}
		} else if queryType == "today" {
			beginTime = now.AddDate(0, 0, 0)
			beginTime = time.Date(beginTime.Year(), beginTime.Month(), beginTime.Day(), 0, 0, 0, 0, now.Location())
			endTime = now
			dayCloudbrainInfo, count, err = getDayCloudbrainInfo(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getDayCloudbrainInfo.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getDayCloudbrainInfo_get_error"))
				return
			}

		} else if queryType == "yesterday" {
			beginTime = now.AddDate(0, 0, -1)
			beginTime = time.Date(beginTime.Year(), beginTime.Month(), beginTime.Day(), 0, 0, 0, 0, now.Location())
			endTime = time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location())
			dayCloudbrainInfo, count, err = getDayCloudbrainInfo(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getDayCloudbrainInfo.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getDayCloudbrainInfo_get_error"))
				return
			}

		} else if queryType == "last_7day" {
			beginTime = now.AddDate(0, 0, -6)
			beginTime = time.Date(beginTime.Year(), beginTime.Month(), beginTime.Day(), 0, 0, 0, 0, now.Location())
			endTime = now
			endTimeTemp = time.Date(endTimeTemp.Year(), endTimeTemp.Month(), endTimeTemp.Day(), 0, 0, 0, 0, now.Location())
			dayCloudbrainInfo, count, err = getDayCloudbrainInfo(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getDayCloudbrainInfo.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getDayCloudbrainInfo_get_error"))
				return
			}
		} else if queryType == "last_30day" {
			beginTime = now.AddDate(0, 0, -29)
			beginTime = time.Date(beginTime.Year(), beginTime.Month(), beginTime.Day(), 0, 0, 0, 0, now.Location())
			endTime = now
			dayCloudbrainInfo, count, err = getDayCloudbrainInfo(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getDayCloudbrainInfo.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getDayCloudbrainInfo_get_error"))
				return
			}
		} else if queryType == "current_month" {
			endTime = now
			beginTime = time.Date(endTime.Year(), endTime.Month(), 1, 0, 0, 0, 0, now.Location())
			dayCloudbrainInfo, count, err = getDayCloudbrainInfo(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getDayCloudbrainInfo.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getDayCloudbrainInfo_get_error"))
				return
			}

		} else if queryType == "current_year" {
			endTime = now
			beginTime = time.Date(endTime.Year(), 1, 1, 0, 0, 0, 0, now.Location())
			dayCloudbrainInfo, count, err = getMonthCloudbrainInfo(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getDayCloudbrainInfo.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getDayCloudbrainInfo_get_error"))
				return
			}

		} else if queryType == "last_month" {

			lastMonthTime := now.AddDate(0, -1, 0)
			beginTime = time.Date(lastMonthTime.Year(), lastMonthTime.Month(), 1, 0, 0, 0, 0, now.Location())
			endTime = time.Date(now.Year(), now.Month(), 1, 0, 0, 0, 0, now.Location())
			dayCloudbrainInfo, count, err = getDayCloudbrainInfo(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getDayCloudbrainInfo.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getDayCloudbrainInfo_get_error"))
				return
			}

		}

	} else {
		if beginTimeStr == "" || endTimeStr == "" {
			//如果查询类型和开始时间结束时间都未设置，按queryType=all处理
			recordCloudbrain, err := models.GetRecordBeginTime()
			if err != nil {
				log.Error("Can not get recordCloudbrain", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("repo.record_begintime_get_err"))
				return
			}
			brainRecordBeginTime := recordCloudbrain[0].Cloudbrain.CreatedUnix.AsTime()
			beginTime = brainRecordBeginTime
			endTime = now
			dayCloudbrainInfo, count, err = getMonthCloudbrainInfo(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getDayCloudbrainInfo.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getDayCloudbrainInfo_get_error"))
				return
			}
		} else {
			beginTime, err = time.ParseInLocation("2006-01-02", beginTimeStr, time.Local)
			if err != nil {
				log.Error("Can not ParseInLocation.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("ParseInLocation_get_error"))
				return
			}
			endTime, err = time.ParseInLocation("2006-01-02", endTimeStr, time.Local)
			if err != nil {
				log.Error("Can not ParseInLocation.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("ParseInLocation_get_error"))
				return
			}
			if endTime.After(time.Now()) {
				endTime = time.Now()
			}
			endTimeTemp = beginTime.AddDate(0, 0, 1)
			dayCloudbrainInfo, count, err = getDayCloudbrainInfo(beginTime, endTime)
			if err != nil {
				log.Error("Can not query getDayCloudbrainInfo.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("getDayCloudbrainInfo_get_error"))
				return
			}
		}

	}

	page := ctx.QueryInt("page")
	if page <= 0 {
		page = 1
	}
	pagesize := ctx.QueryInt("pagesize")
	if pagesize <= 0 {
		pagesize = 5
	}
	pageDateCloudbrainInfo := getPageDateCloudbrainInfo(dayCloudbrainInfo, page, pagesize)

	ctx.JSON(http.StatusOK, map[string]interface{}{
		"totalCount":         count,
		"timeCloudbrainInfo": pageDateCloudbrainInfo,
	})

}
func getPageDateCloudbrainInfo(dateCloudbrainInfo []DateCloudbrainInfo, page int, pagesize int) []DateCloudbrainInfo {
	begin := (page - 1) * pagesize
	end := (page) * pagesize

	if begin > len(dateCloudbrainInfo)-1 {
		return nil
	}
	if end > len(dateCloudbrainInfo)-1 {
		return dateCloudbrainInfo[begin:]
	} else {
		return dateCloudbrainInfo[begin:end]
	}

}

func GetAllCloudbrainsPeriodDistribution(ctx *context.Context) {
	queryType := ctx.QueryTrim("type")
	beginTimeStr := ctx.QueryTrim("beginTime")
	endTimeStr := ctx.QueryTrim("endTime")
	recordCloudbrain, err := models.GetRecordBeginTime()
	if err != nil {
		log.Error("Can not get record begin time", err)
		ctx.Error(http.StatusBadRequest, ctx.Tr("repo.record_begintime_get_err"))
		return
	}
	recordBeginTime := time.Unix(int64(recordCloudbrain[0].Cloudbrain.CreatedUnix), 0)
	beginTime, endTime, err := getCloudbrainTimePeroid(ctx, recordBeginTime)
	if err != nil {
		log.Error("Parameter is wrong", err)
		ctx.Error(http.StatusBadRequest, ctx.Tr("repo.parameter_is_wrong"))
		return
	}
	cloudOneJobTypeRes := make(map[string]int)
	cloudTwoJobTypeRes := make(map[string]int)
	intelligentNetJobTypeRes := make(map[string]int)
	cloudBrainPeriodNum := make(map[int]int)
	cloudBrainComputeResource := make(map[string]int)
	beginTimeTemp := beginTime.Unix()
	if queryType == "all" || (queryType == "" && (beginTimeStr == "" || endTimeStr == "")) {
		beginTimeTemp = int64(recordCloudbrain[0].Cloudbrain.CreatedUnix)
	}

	page := 1
	pagesize := 1000
	count := pagesize
	//Each time a maximum of 1000 pieces of data are detected to the memory, batch processing
	for count == pagesize && count != 0 {
		cloudbrains, _, err := models.CloudbrainAllStatic(&models.CloudbrainsOptions{
			ListOptions: models.ListOptions{
				Page:     page,
				PageSize: pagesize,
			},
			Type:          models.TypeCloudBrainAll,
			BeginTimeUnix: beginTimeTemp,
			EndTimeUnix:   endTime.Unix(),
		})
		if err != nil {
			ctx.ServerError("Get cloudbrains failed:", err)
			return
		}

		for _, cloudbrain := range cloudbrains {
			if cloudbrain.Cloudbrain.Type == models.TypeCloudBrainOne {
				if _, ok := cloudOneJobTypeRes[cloudbrain.JobType]; !ok {
					cloudOneJobTypeRes[cloudbrain.JobType] = 1
				} else {
					cloudOneJobTypeRes[cloudbrain.JobType] += 1
				}
			}
			if cloudbrain.Cloudbrain.Type == models.TypeCloudBrainTwo {
				if _, ok := cloudTwoJobTypeRes[cloudbrain.JobType]; !ok {
					cloudTwoJobTypeRes[cloudbrain.JobType] = 1
				} else {
					cloudTwoJobTypeRes[cloudbrain.JobType] += 1
				}
			}
			if cloudbrain.Cloudbrain.Type == models.TypeC2Net {
				if _, ok := intelligentNetJobTypeRes[cloudbrain.JobType]; !ok {
					intelligentNetJobTypeRes[cloudbrain.JobType] = 1
				} else {
					intelligentNetJobTypeRes[cloudbrain.JobType] += 1
				}
			}

			if _, ok := cloudBrainPeriodNum[cloudbrain.Cloudbrain.Type]; !ok {
				cloudBrainPeriodNum[cloudbrain.Cloudbrain.Type] = 1
			} else {
				cloudBrainPeriodNum[cloudbrain.Cloudbrain.Type] += 1
			}

			if _, ok := cloudBrainComputeResource[cloudbrain.Cloudbrain.ComputeResource]; !ok {
				cloudBrainComputeResource[cloudbrain.Cloudbrain.ComputeResource] = 1
			} else {
				cloudBrainComputeResource[cloudbrain.Cloudbrain.ComputeResource] += 1
			}
		}
		count = len(cloudbrains)
		page += 1
	}

	jobTypeList := []string{string(models.JobTypeDebug), string(models.JobTypeTrain), string(models.JobTypeInference), string(models.JobTypeBenchmark),
		string(models.JobTypeBrainScore), string(models.JobTypeSnn4imagenet)}
	for _, v := range jobTypeList {
		if _, ok := cloudOneJobTypeRes[v]; !ok {
			cloudOneJobTypeRes[v] = 0
		}
		if _, ok := cloudTwoJobTypeRes[v]; !ok {
			cloudTwoJobTypeRes[v] = 0
		}
		if _, ok := intelligentNetJobTypeRes[v]; !ok {
			intelligentNetJobTypeRes[v] = 0
		}
	}
	cloudBrainTypeList := []int{0, 1, 2}
	for _, v := range cloudBrainTypeList {
		if _, ok := cloudBrainPeriodNum[v]; !ok {
			cloudBrainPeriodNum[v] = 0
		}
	}

	ComputeResourceList := []string{"CPU/GPU", "NPU"}
	for _, v := range ComputeResourceList {
		if _, ok := cloudBrainComputeResource[v]; !ok {
			cloudBrainComputeResource[v] = 0
		}
	}

	cloudOneJobTypeRes["EVALUATION"] = cloudOneJobTypeRes[string(models.JobTypeBenchmark)] + cloudOneJobTypeRes[string(models.JobTypeSnn4imagenet)] + cloudOneJobTypeRes[string(models.JobTypeBrainScore)]
	cloudTwoJobTypeRes["EVALUATION"] = cloudTwoJobTypeRes[string(models.JobTypeBenchmark)] + cloudTwoJobTypeRes[string(models.JobTypeSnn4imagenet)] + cloudTwoJobTypeRes[string(models.JobTypeBrainScore)]
	intelligentNetJobTypeRes["EVALUATION"] = intelligentNetJobTypeRes[string(models.JobTypeBenchmark)] + intelligentNetJobTypeRes[string(models.JobTypeSnn4imagenet)] + intelligentNetJobTypeRes[string(models.JobTypeBrainScore)]
	ctx.JSON(http.StatusOK, map[string]interface{}{
		"cloudOneJobTypeRes":        cloudOneJobTypeRes,
		"cloudTwoJobTypeRes":        cloudTwoJobTypeRes,
		"intelligentNetJobTypeRes":  intelligentNetJobTypeRes,
		"cloudBrainPeriodNum":       cloudBrainPeriodNum,
		"cloudBrainComputeResource": cloudBrainComputeResource,
	})
}

func GetCloudbrainsStatusAnalysis(ctx *context.Context) {
	recordCloudbrain, err := models.GetRecordBeginTime()
	if err != nil {
		log.Error("Can not get recordCloudbrain", err)
		ctx.Error(http.StatusBadRequest, ctx.Tr("repo.record_begintime_get_err"))
		return
	}
	recordBeginTime := recordCloudbrain[0].Cloudbrain.CreatedUnix
	endTime := time.Now()
	cloudBrainStatusResult := make(map[string]int)
	cloudBrainStatusAnalysis := make(map[string]int)
	totalCount := 0

	page := 1
	pagesize := 1000
	count := pagesize
	for count == pagesize && count != 0 {
		cloudbrains, _, err := models.CloudbrainAllStatic(&models.CloudbrainsOptions{
			ListOptions: models.ListOptions{
				Page:     page,
				PageSize: pagesize,
			},
			Type:          models.TypeCloudBrainAll,
			BeginTimeUnix: int64(recordBeginTime),
			EndTimeUnix:   endTime.Unix(),
		})
		if err != nil {
			ctx.ServerError("Get cloudbrains failed:", err)
			return
		}
		for _, cloudbrain := range cloudbrains {
			if _, ok := cloudBrainStatusResult[cloudbrain.Status]; !ok {
				cloudBrainStatusResult[cloudbrain.Status] = 1
			} else {
				cloudBrainStatusResult[cloudbrain.Status] += 1
			}
		}
		count = len(cloudbrains)
		totalCount = totalCount + count
		page += 1
	}

	var jobStatuses []string
	jobStatuses = append(jobStatuses, string(models.ModelArtsTrainJobWaiting), string(models.ModelArtsTrainJobFailed), string(models.ModelArtsRunning), string(models.ModelArtsTrainJobCompleted),
		string(models.ModelArtsStarting), string(models.ModelArtsRestarting), string(models.ModelArtsStartFailed),
		string(models.ModelArtsStopping), string(models.ModelArtsStopped), string(models.JobSucceeded))
	jobStatusesCount := 0
	for _, v := range jobStatuses {
		if _, ok := cloudBrainStatusResult[v]; !ok {
			cloudBrainStatusAnalysis[v] = 0
		} else {
			cloudBrainStatusAnalysis[v] = cloudBrainStatusResult[v]
		}
		jobStatusesCount = jobStatusesCount + cloudBrainStatusResult[v]
	}
	cloudBrainStatusAnalysis["OTHER"] = totalCount - jobStatusesCount

	ctx.JSON(http.StatusOK, map[string]interface{}{
		"cloudBrainStatusResult": cloudBrainStatusAnalysis,
	})
}

func GetCloudbrainsDetailData(ctx *context.Context) {
	recordCloudbrain, err := models.GetRecordBeginTime()
	if err != nil {
		log.Error("Can not get recordCloudbrain", err)
		ctx.Error(http.StatusBadRequest, ctx.Tr("repo.record_begintime_get_err"))
		return
	}
	recordBeginTime := recordCloudbrain[0].Cloudbrain.CreatedUnix
	endTime := time.Now()
	listType := ctx.Query("listType")
	jobType := ctx.Query("jobType")
	jobStatus := ctx.Query("jobStatus")
	cloudBrainType := ctx.QueryInt("Type")

	page := ctx.QueryInt("page")
	pageSize := ctx.QueryInt("pagesize")
	if page <= 0 {
		page = 1
	}
	if pageSize <= 0 {
		pageSize = 10
	}
	var jobTypes []string
	jobTypeNot := false
	if jobType == string(models.JobTypeBenchmark) {
		jobTypes = append(jobTypes, string(models.JobTypeBenchmark), string(models.JobTypeBrainScore), string(models.JobTypeSnn4imagenet))
	} else if jobType != "all" && jobType != "" {
		jobTypes = append(jobTypes, jobType)
	}

	var jobStatuses []string
	jobStatusNot := false
	if jobStatus == "other" {
		jobStatusNot = true
		jobStatuses = append(jobStatuses, string(models.ModelArtsTrainJobWaiting), string(models.ModelArtsTrainJobFailed), string(models.ModelArtsRunning), string(models.ModelArtsTrainJobCompleted),
			string(models.ModelArtsStarting), string(models.ModelArtsRestarting), string(models.ModelArtsStartFailed),
			string(models.ModelArtsStopping), string(models.ModelArtsStopped), string(models.JobSucceeded))
	} else if jobStatus != "all" && jobStatus != "" {
		jobStatuses = append(jobStatuses, jobStatus)
	}

	keyword := strings.Trim(ctx.Query("q"), " ")

	ciTasks, count, err := models.CloudbrainAll(&models.CloudbrainsOptions{
		ListOptions: models.ListOptions{
			Page:     page,
			PageSize: pageSize,
		},
		Keyword:         keyword,
		Type:            cloudBrainType,
		ComputeResource: listType,
		JobTypeNot:      jobTypeNot,
		JobStatusNot:    jobStatusNot,
		JobStatus:       jobStatuses,
		JobTypes:        jobTypes,
		NeedRepoInfo:    true,
		BeginTimeUnix:   int64(recordBeginTime),
		EndTimeUnix:     endTime.Unix(),
	})
	if err != nil {
		ctx.ServerError("Get job failed:", err)
		return
	}
	nilTime := time.Time{}
	tasks := []models.TaskDetail{}
	for i, task := range ciTasks {
		ciTasks[i].Cloudbrain.ComputeResource = task.ComputeResource

		var taskDetail models.TaskDetail
		taskDetail.ID = ciTasks[i].Cloudbrain.ID
		taskDetail.JobName = ciTasks[i].JobName
		taskDetail.DisplayJobName = ciTasks[i].DisplayJobName
		taskDetail.Status = ciTasks[i].Status
		taskDetail.JobType = ciTasks[i].JobType
		taskDetail.CreatedUnix = ciTasks[i].Cloudbrain.CreatedUnix
		taskDetail.RunTime = ciTasks[i].Cloudbrain.TrainJobDuration
		taskDetail.StartTime = ciTasks[i].StartTime
		taskDetail.EndTime = ciTasks[i].EndTime
		taskDetail.ComputeResource = ciTasks[i].ComputeResource
		taskDetail.Type = ciTasks[i].Cloudbrain.Type
		taskDetail.UserName = ciTasks[i].User.Name
		taskDetail.RepoID = ciTasks[i].RepoID
		if ciTasks[i].Repo != nil {
			taskDetail.RepoName = ciTasks[i].Repo.OwnerName + "/" + ciTasks[i].Repo.Name
			taskDetail.RepoAlias = ciTasks[i].Repo.OwnerName + "/" + ciTasks[i].Repo.Alias
		}
		if ciTasks[i].Cloudbrain.Status == string(models.JobWaiting) {
			if ciTasks[i].Cloudbrain.DeletedAt != nilTime {
				WaitTimeInt := ciTasks[i].Cloudbrain.UpdatedUnix.AsTime().Unix() - ciTasks[i].Cloudbrain.CreatedUnix.AsTime().Unix()
				taskDetail.WaitTime = models.ConvertDurationToStr(WaitTimeInt)
				if WaitTimeInt < 0 {
					taskDetail.WaitTime = "00:00:00"
				}
			} else {
				if ciTasks[i].Cloudbrain.StartTime.AsTime().Unix() == 0 {
					WaitTimeInt := time.Now().Unix() - ciTasks[i].Cloudbrain.CreatedUnix.AsTime().Unix()
					taskDetail.WaitTime = models.ConvertDurationToStr(WaitTimeInt)
					if WaitTimeInt < 0 {
						taskDetail.WaitTime = "00:00:00"
					}
				} else {
					WaitTimeInt := ciTasks[i].Cloudbrain.StartTime.AsTime().Unix() - ciTasks[i].Cloudbrain.CreatedUnix.AsTime().Unix()
					taskDetail.WaitTime = models.ConvertDurationToStr(WaitTimeInt)
					if WaitTimeInt < 0 {
						taskDetail.WaitTime = "00:00:00"
					}
				}
			}
		} else if ciTasks[i].Cloudbrain.Status == string(models.JobStopped) && ciTasks[i].Cloudbrain.StartTime.AsTime().Unix() == 0 {
			WaitTimeInt := ciTasks[i].Cloudbrain.EndTime.AsTime().Unix() - ciTasks[i].Cloudbrain.CreatedUnix.AsTime().Unix()
			taskDetail.WaitTime = models.ConvertDurationToStr(WaitTimeInt)
			if WaitTimeInt < 0 {
				taskDetail.WaitTime = "00:00:00"

			}
		} else {
			WaitTimeInt := ciTasks[i].Cloudbrain.StartTime.AsTime().Unix() - ciTasks[i].Cloudbrain.CreatedUnix.AsTime().Unix()
			taskDetail.WaitTime = models.ConvertDurationToStr(WaitTimeInt)
			if WaitTimeInt < 0 {
				taskDetail.WaitTime = "00:00:00"
			}
		}

		if ciTasks[i].Cloudbrain.Type == models.TypeCloudBrainTwo || (ciTasks[i].Cloudbrain.Type == models.TypeCloudBrainOne && ciTasks[i].Cloudbrain.JobType == "TRAIN") {
			taskDetail.JobID = ciTasks[i].Cloudbrain.JobID
		}

		if ciTasks[i].Cloudbrain.DeletedAt != nilTime {
			taskDetail.IsDelete = true
		} else {
			taskDetail.IsDelete = false
		}
		tasks = append(tasks, taskDetail)
	}

	pager := context.NewPagination(int(count), pageSize, page, getTotalPage(count, pageSize))
	pager.SetDefaultParams(ctx)
	pager.AddParam(ctx, "listType", "ListType")

	ctx.JSON(http.StatusOK, map[string]interface{}{
		"Title":   ctx.Tr("kanban.cloudBrains"),
		"Tasks":   tasks,
		"Keyword": keyword,
		"pager":   pager,
		"count":   count,
	})
}

func GetCloudbrainsCreateHoursData(ctx *context.Context) {
	recordCloudbrain, err := models.GetRecordBeginTime()
	if err != nil {
		log.Error("Can not get record begin time", err)
		ctx.Error(http.StatusBadRequest, ctx.Tr("repo.record_begintime_get_err"))
		return
	}
	recordBeginTime := time.Unix(int64(recordCloudbrain[0].Cloudbrain.CreatedUnix), 0)
	// recordBeginTime, _ := time.Parse("2006-01-02 15:04:05", recordTime)
	now := time.Now()
	queryType := ctx.QueryTrim("type")
	beginTimeStr := ctx.QueryTrim("beginTime")
	endTimeStr := ctx.QueryTrim("endTime")
	var beginTime time.Time
	var endTime time.Time
	createHourPeriodCount := make(map[string]interface{})
	if queryType != "" {
		if queryType == "all" {
			beginTime = recordBeginTime
			endTime = now.AddDate(0, 0, 1)
			endTime = time.Date(endTime.Year(), endTime.Month(), endTime.Day(), 0, 0, 0, 0, now.Location())
		} else if queryType == "today" {
			beginTime = time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location())
			endTime = now.AddDate(0, 0, 1)
		} else if queryType == "yesterday" {
			beginTime = now.AddDate(0, 0, -1)
			beginTime = time.Date(beginTime.Year(), beginTime.Month(), beginTime.Day(), 0, 0, 0, 0, now.Location())
			endTime = time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location())
		} else if queryType == "current_week" {
			beginTime = now.AddDate(0, 0, -int(time.Now().Weekday())+1) //begin from monday
			endTime = now
		} else if queryType == "current_month" {
			beginTime = time.Date(now.Year(), now.Month(), 1, 0, 0, 0, 0, now.Location())
			endTime = now.AddDate(0, 0, 1)
			endTime = time.Date(endTime.Year(), endTime.Month(), endTime.Day(), 0, 0, 0, 0, now.Location())
		} else if queryType == "monthly" {
			endTime = now
			beginTime = now.AddDate(0, -1, 0)
			beginTime = time.Date(beginTime.Year(), beginTime.Month(), beginTime.Day(), 0, 0, 0, 0, now.Location())
		} else if queryType == "current_year" {
			beginTime = time.Date(now.Year(), 1, 1, 0, 0, 0, 0, now.Location())
			endTime = now.AddDate(0, 0, 1)
			endTime = time.Date(endTime.Year(), endTime.Month(), endTime.Day(), 0, 0, 0, 0, now.Location())
		} else if queryType == "last_month" {
			lastMonthTime := now.AddDate(0, -1, 0)
			beginTime = time.Date(lastMonthTime.Year(), lastMonthTime.Month(), 1, 0, 0, 0, 0, now.Location())
			endTime = time.Date(now.Year(), now.Month(), 1, 0, 0, 0, 0, now.Location())
		} else if queryType == "last_7day" {
			beginTime = now.AddDate(0, 0, -6)
			beginTime = time.Date(beginTime.Year(), beginTime.Month(), beginTime.Day(), 0, 0, 0, 0, now.Location())
			endTime = now.AddDate(0, 0, 1)
			endTime = time.Date(endTime.Year(), endTime.Month(), endTime.Day(), 0, 0, 0, 0, now.Location())
		} else if queryType == "last_30day" {
			beginTime = now.AddDate(0, 0, -29)
			beginTime = time.Date(beginTime.Year(), beginTime.Month(), beginTime.Day(), 0, 0, 0, 0, now.Location())
			endTime = now.AddDate(0, 0, 1)
			endTime = time.Date(endTime.Year(), endTime.Month(), endTime.Day(), 0, 0, 0, 0, now.Location())
		}
	} else {
		if beginTimeStr == "" || endTimeStr == "" {
			//如果查询类型和开始时间结束时间都未设置，按queryType=all处理
			beginTime = recordBeginTime
			endTime = now.AddDate(0, 0, 1)
			endTime = time.Date(endTime.Year(), endTime.Month(), endTime.Day(), 0, 0, 0, 0, now.Location())
		} else {
			beginTime, err = time.ParseInLocation("2006-01-02", beginTimeStr, time.Local)
			if err != nil {
				log.Error("Can not ParseInLocation.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("ParseInLocation_get_error"))
				return
			}
			endTime, err = time.ParseInLocation("2006-01-02", endTimeStr, time.Local)
			if err != nil {
				log.Error("Can not ParseInLocation.", err)
				ctx.Error(http.StatusBadRequest, ctx.Tr("ParseInLocation_get_error"))
				return
			}
			endTime = time.Date(endTime.Year(), endTime.Month(), endTime.Day(), 0, 0, 0, 0, now.Location())
		}
	}
	dateBeginTime := beginTime.Format("2006-01-02")
	dateEndTime := endTime.Format("2006-01-02")
	createHourPeriodCount, err = models.GetCreateHourPeriodCount(dateBeginTime, dateEndTime)
	if err != nil {
		log.Error("Can not query hourPeriodCount.", err)
		ctx.Error(http.StatusBadRequest, ctx.Tr("hourPeriodCount_get_error"))
		return
	}

	runHourPeriodCount, err := models.GetRunHourPeriodCount(dateBeginTime, dateEndTime)
	if err != nil {
		log.Error("Can not query runHourPeriodCount.", err)
		ctx.Error(http.StatusBadRequest, ctx.Tr("runHourPeriodCount_get_error"))
		return
	}
	ctx.JSON(http.StatusOK, map[string]interface{}{
		"recordBeginTime":       recordCloudbrain[0].Cloudbrain.CreatedUnix,
		"updateTime":            now.Unix(),
		"createHourPeriodCount": createHourPeriodCount,
		"runHourPeriodCount":    runHourPeriodCount,
	})

}
func GetWaittingTop(ctx *context.Context) {
	ciTasks, err := models.GetWaittingTop()
	if err != nil {
		log.Error("Can not get record begin time", err)
		ctx.Error(http.StatusBadRequest, ctx.Tr("repo.record_begintime_get_err"))
		return
	}
	tasks := []models.TaskDetail{}
	for i, task := range ciTasks {
		ciTasks[i].Cloudbrain.ComputeResource = task.ComputeResource

		var taskDetail models.TaskDetail
		taskDetail.DisplayJobName = ciTasks[i].DisplayJobName
		taskDetail.UserName = ciTasks[i].User.Name
		taskDetail.RepoID = ciTasks[i].RepoID
		if ciTasks[i].Repo != nil {
			taskDetail.RepoName = ciTasks[i].Repo.OwnerName + "/" + ciTasks[i].Repo.Name
		}
		WaitTimeInt := time.Now().Unix() - ciTasks[i].Cloudbrain.CreatedUnix.AsTime().Unix()
		taskDetail.WaitTime = models.ConvertDurationToStr(WaitTimeInt)

		if WaitTimeInt < 0 {
			taskDetail.WaitTime = "00:00:00"
		}
		tasks = append(tasks, taskDetail)
	}
	ctx.JSON(http.StatusOK, map[string]interface{}{
		"tasks": tasks,
	})
}
func GetRunningTop(ctx *context.Context) {
	ciTasks, err := models.GetRunningTop()
	if err != nil {
		log.Error("Can not get record begin time", err)
		ctx.Error(http.StatusBadRequest, ctx.Tr("repo.record_begintime_get_err"))
		return
	}
	tasks := []models.TaskDetail{}
	for i, task := range ciTasks {
		ciTasks[i].Cloudbrain.ComputeResource = task.ComputeResource

		var taskDetail models.TaskDetail
		taskDetail.DisplayJobName = ciTasks[i].DisplayJobName
		taskDetail.RunTime = ciTasks[i].Cloudbrain.TrainJobDuration
		taskDetail.UserName = ciTasks[i].User.Name
		taskDetail.RepoID = ciTasks[i].RepoID
		if ciTasks[i].Repo != nil {
			taskDetail.RepoName = ciTasks[i].Repo.OwnerName + "/" + ciTasks[i].Repo.Name
		}

		tasks = append(tasks, taskDetail)
	}
	ctx.JSON(http.StatusOK, map[string]interface{}{
		"tasks": tasks,
	})
}

func getCloudbrainCount(beginTime time.Time, endTime time.Time, cloudbrains []*models.CloudbrainInfo) (map[string]int, map[string]int, map[string]int, map[int]int, map[string]int) {

	cloudOneJobTypeRes := make(map[string]int)
	cloudTwoJobTypeRes := make(map[string]int)
	intelligentNetJobTypeRes := make(map[string]int)
	cloudBrainPeriodNum := make(map[int]int)
	cloudBrainComputeResource := make(map[string]int)
	for _, cloudbrain := range cloudbrains {
		if int64(cloudbrain.Cloudbrain.CreatedUnix) >= beginTime.Unix() && int64(cloudbrain.Cloudbrain.CreatedUnix) < endTime.Unix() {
			if cloudbrain.Cloudbrain.Type == models.TypeCloudBrainOne {
				if _, ok := cloudOneJobTypeRes[cloudbrain.Cloudbrain.JobType]; !ok {
					cloudOneJobTypeRes[cloudbrain.Cloudbrain.JobType] = 1
				} else {
					cloudOneJobTypeRes[cloudbrain.Cloudbrain.JobType] += 1
				}
			}
			if cloudbrain.Cloudbrain.Type == models.TypeCloudBrainTwo {
				if _, ok := cloudTwoJobTypeRes[cloudbrain.JobType]; !ok {
					cloudTwoJobTypeRes[cloudbrain.Cloudbrain.JobType] = 1
				} else {
					cloudTwoJobTypeRes[cloudbrain.Cloudbrain.JobType] += 1
				}
			}

			if _, ok := cloudBrainPeriodNum[cloudbrain.Cloudbrain.Type]; !ok {
				cloudBrainPeriodNum[cloudbrain.Cloudbrain.Type] = 1
			} else {
				cloudBrainPeriodNum[cloudbrain.Cloudbrain.Type] += 1
			}

			if _, ok := cloudBrainComputeResource[cloudbrain.Cloudbrain.ComputeResource]; !ok {
				cloudBrainComputeResource[cloudbrain.Cloudbrain.ComputeResource] = 1
			} else {
				cloudBrainComputeResource[cloudbrain.Cloudbrain.ComputeResource] += 1
			}

		}
	}

	ComputeResourceList := []string{"CPU/GPU", "NPU"}
	for _, v := range ComputeResourceList {
		if _, ok := cloudBrainComputeResource[v]; !ok {
			cloudBrainComputeResource[v] = 0
		}
	}

	jobTypeList := []string{"DEBUG", "BENCHMARK", "INFERENCE", "TRAIN", "SNN4IMAGENET", "BRAINSCORE"}
	cloudBrainTypeList := []int{0, 1, 2}
	for _, v := range jobTypeList {
		if _, ok := cloudOneJobTypeRes[v]; !ok {
			cloudOneJobTypeRes[v] = 0
		}
		if _, ok := cloudTwoJobTypeRes[v]; !ok {
			cloudTwoJobTypeRes[v] = 0
		}
		if _, ok := intelligentNetJobTypeRes[v]; !ok {
			intelligentNetJobTypeRes[v] = 0
		}
	}
	for _, v := range cloudBrainTypeList {
		if _, ok := cloudBrainPeriodNum[v]; !ok {
			cloudBrainPeriodNum[v] = 0
		}
	}
	cloudBrainPeriodNum[-1] = cloudBrainPeriodNum[0] + cloudBrainPeriodNum[1] + cloudBrainPeriodNum[2]
	return cloudOneJobTypeRes, cloudTwoJobTypeRes, intelligentNetJobTypeRes, cloudBrainPeriodNum, cloudBrainComputeResource
}

func getDayCloudbrainNum(beginTime time.Time, endTime time.Time) ([]DateCloudbrainNum, error) {
	var endTimeTemp time.Time
	endTimeTemp = beginTime.AddDate(0, 0, 1)
	cloudbrains, _, err := models.CloudbrainAllStatic(&models.CloudbrainsOptions{
		Type:          models.TypeCloudBrainAll,
		BeginTimeUnix: beginTime.Unix(),
		EndTimeUnix:   endTime.Unix(),
	})
	if err != nil {
		log.Error("Get cloudbrains failed:", err)
		return nil, err
	}
	dayCloudbrainNum := make([]DateCloudbrainNum, 0)
	for endTimeTemp.Before(endTime) || endTimeTemp.Equal(endTime) {
		cloudOneJobTypeRes, cloudTwoJobTypeRes, intelligentNetJobTypeRes, cloudBrainPeriodNum, cloudBrainComputeResource := getCloudbrainCount(beginTime, endTimeTemp, cloudbrains)
		dayCloudbrainNum = append(dayCloudbrainNum, DateCloudbrainNum{
			Date:                      beginTime.Format("2006/01/02"),
			CloudOneJobTypeRes:        cloudOneJobTypeRes,
			CloudTwoJobTypeRes:        cloudTwoJobTypeRes,
			IntelligentNetJobTypeRes:  intelligentNetJobTypeRes,
			CloudBrainPeriodNum:       cloudBrainPeriodNum,
			CloudBrainComputeResource: cloudBrainComputeResource,
		})
		if endTime.Before(endTimeTemp.AddDate(0, 0, 1)) && endTimeTemp.Before(endTime) {
			beginTime = endTimeTemp
			endTimeTemp = endTime
		} else {
			beginTime = endTimeTemp
			endTimeTemp = beginTime.AddDate(0, 0, 1)
		}
	}
	return dayCloudbrainNum, nil
}
func getMonthCloudbrainNum(beginTime time.Time, endTime time.Time) ([]DateCloudbrainNum, error) {
	var endTimeTemp time.Time
	now := time.Now()
	endTimeTemp = beginTime.AddDate(0, 1, 0)
	endTimeTemp = time.Date(endTimeTemp.Year(), endTimeTemp.Month(), 1, 0, 0, 0, 0, now.Location())
	monthCloudbrainNum := make([]DateCloudbrainNum, 0)
	cloudbrains, _, err := models.CloudbrainAllStatic(&models.CloudbrainsOptions{
		Type:          models.TypeCloudBrainAll,
		BeginTimeUnix: beginTime.Unix(),
		EndTimeUnix:   endTime.Unix(),
	})
	if err != nil {
		log.Error("Getcloudbrains failed:%v", err)
		return nil, err
	}
	for endTimeTemp.Before(endTime) || endTimeTemp.Equal(endTime) {
		cloudOneJobTypeRes, cloudTwoJobTypeRes, intelligentNetJobTypeRes, cloudBrainPeriodNum, cloudBrainComputeResource := getCloudbrainCount(beginTime, endTimeTemp, cloudbrains)
		monthCloudbrainNum = append(monthCloudbrainNum, DateCloudbrainNum{
			Date:                      beginTime.Format("2006/01"),
			CloudOneJobTypeRes:        cloudOneJobTypeRes,
			CloudTwoJobTypeRes:        cloudTwoJobTypeRes,
			IntelligentNetJobTypeRes:  intelligentNetJobTypeRes,
			CloudBrainPeriodNum:       cloudBrainPeriodNum,
			CloudBrainComputeResource: cloudBrainComputeResource,
		})
		if endTime.Before(endTimeTemp.AddDate(0, 1, 0)) && endTimeTemp.Before(endTime) {
			beginTime = endTimeTemp
			endTimeTemp = endTime
		} else {
			beginTime = endTimeTemp
			endTimeTemp = beginTime.AddDate(0, 1, 0)
		}
	}
	return monthCloudbrainNum, nil
}

func getDayCloudbrainInfo(beginTime time.Time, endTime time.Time) ([]DateCloudbrainInfo, int, error) {
	now := time.Now()
	endTimeTemp := time.Date(endTime.Year(), endTime.Month(), endTime.Day(), 0, 0, 0, 0, now.Location())
	if endTimeTemp.Equal(endTime) {
		endTimeTemp = endTimeTemp.AddDate(0, 0, -1)
	}
	cloudbrains, _, err := models.CloudbrainAllStatic(&models.CloudbrainsOptions{
		Type:          models.TypeCloudBrainAll,
		BeginTimeUnix: beginTime.Unix(),
		EndTimeUnix:   endTime.Unix(),
	})
	if err != nil {
		log.Error("Getcloudbrains failed:%v", err)
		return nil, 0, err
	}
	dayCloudbrainInfo := make([]DateCloudbrainInfo, 0)
	count := 0
	for beginTime.Before(endTimeTemp) || beginTime.Equal(endTimeTemp) {
		_, _, _, cloudBrainPeriodNum, cloudBrainComputeResource := getCloudbrainCount(endTimeTemp, endTime, cloudbrains)
		dayCloudbrainInfo = append(dayCloudbrainInfo, DateCloudbrainInfo{
			Date:                      endTimeTemp.Format("2006/01/02"),
			CloudBrainPeriodNum:       cloudBrainPeriodNum,
			CloudBrainComputeResource: cloudBrainComputeResource,
		})
		endTime = endTimeTemp
		endTimeTemp = endTimeTemp.AddDate(0, 0, -1)
		count += 1
	}
	return dayCloudbrainInfo, count, nil
}

func getMonthCloudbrainInfo(beginTime time.Time, endTime time.Time) ([]DateCloudbrainInfo, int, error) {
	now := time.Now()
	endTimeTemp := time.Date(endTime.Year(), endTime.Month(), 1, 0, 0, 0, 0, now.Location())
	if endTimeTemp.Equal(endTime) {
		endTimeTemp = endTimeTemp.AddDate(0, -1, 0)
	}
	cloudbrains, _, err := models.CloudbrainAllStatic(&models.CloudbrainsOptions{
		Type:          models.TypeCloudBrainAll,
		BeginTimeUnix: beginTime.Unix(),
		EndTimeUnix:   endTime.Unix(),
	})
	if err != nil {
		log.Error("Getcloudbrains failed:%v", err)
		return nil, 0, err
	}
	dayCloudbrainInfo := make([]DateCloudbrainInfo, 0)
	count := 0
	for beginTime.Before(endTimeTemp) || beginTime.Equal(endTimeTemp) || (endTimeTemp.Before(beginTime) && beginTime.Before(endTime)) {
		_, _, _, cloudBrainPeriodNum, cloudBrainComputeResource := getCloudbrainCount(endTimeTemp, endTime, cloudbrains)
		dayCloudbrainInfo = append(dayCloudbrainInfo, DateCloudbrainInfo{
			Date:                      endTimeTemp.Format("2006/01"),
			CloudBrainPeriodNum:       cloudBrainPeriodNum,
			CloudBrainComputeResource: cloudBrainComputeResource,
		})
		endTime = endTimeTemp
		endTimeTemp = endTimeTemp.AddDate(0, -1, 0)
		count += 1
	}
	return dayCloudbrainInfo, count, nil
}

func DownloadCloudBrainBoard(ctx *context.Context) {

	page := 1

	pageSize := 300

	var cloudBrain = ctx.Tr("repo.cloudbrain")
	fileName := getCloudbrainFileName(cloudBrain)

	_, total, err := models.CloudbrainAll(&models.CloudbrainsOptions{
		ListOptions: models.ListOptions{
			Page:     page,
			PageSize: pageSize,
		},
		Type:         models.TypeCloudBrainAll,
		NeedRepoInfo: false,
	})

	if err != nil {
		log.Warn("Can not get cloud brain info", err)
		ctx.Error(http.StatusBadRequest, ctx.Tr("repo.cloudbrain_query_fail"))
		return
	}
	totalPage := getTotalPage(total, pageSize)
	f := excelize.NewFile()

	index := f.NewSheet(cloudBrain)
	f.DeleteSheet("Sheet1")

	for k, v := range allCloudbrainHeader(ctx) {
		f.SetCellValue(cloudBrain, k, v)
	}

	var row = 2
	for i := 0; i < totalPage; i++ {

		pageRecords, _, err := models.CloudbrainAll(&models.CloudbrainsOptions{
			ListOptions: models.ListOptions{
				Page:     page,
				PageSize: pageSize,
			},
			Type:         models.TypeCloudBrainAll,
			NeedRepoInfo: true,
		})
		if err != nil {
			log.Warn("Can not get cloud brain info", err)
			continue
		}
		for _, record := range pageRecords {

			for k, v := range allCloudbrainValues(row, record, ctx) {
				f.SetCellValue(cloudBrain, k, v)
			}
			row++

		}

		page++
	}
	f.SetActiveSheet(index)

	ctx.Resp.Header().Set("Content-Disposition", "attachment; filename="+url.QueryEscape(fileName))
	ctx.Resp.Header().Set("Content-Type", "application/octet-stream")

	f.WriteTo(ctx.Resp)
}

func getCloudbrainFileName(baseName string) string {
	return baseName + "_" + time.Now().Format(EXCEL_DATE_FORMAT) + ".xlsx"

}
func allCloudbrainHeader(ctx *context.Context) map[string]string {

	return map[string]string{"A1": ctx.Tr("repo.cloudbrain_task"), "B1": ctx.Tr("repo.cloudbrain_type"), "C1": ctx.Tr("repo.modelarts.status"), "D1": ctx.Tr("repo.cloudbrain_task_type"),
		"E1": ctx.Tr("repo.modelarts.createtime"), "F1": ctx.Tr("repo.modelarts.train_job.wait_time"), "G1": ctx.Tr("repo.modelarts.train_job.dura_time"),
		"H1": ctx.Tr("repo.modelarts.train_job.start_time"),
		"I1": ctx.Tr("repo.modelarts.train_job.end_time"), "J1": ctx.Tr("repo.modelarts.computing_resources"),
		"K1": ctx.Tr("repo.cloudbrain_creator"), "L1": ctx.Tr("repo.repo_name"), "M1": ctx.Tr("repo.cloudbrain_task_name"), "N1": ctx.Tr("repo.modelarts.deletetime")}

}
func allCloudbrainValues(row int, rs *models.CloudbrainInfo, ctx *context.Context) map[string]string {
	return map[string]string{getCellName("A", row): rs.DisplayJobName, getCellName("B", row): getCloudbrainType(rs, ctx), getCellName("C", row): rs.Status, getCellName("D", row): rs.JobType,
		getCellName("E", row): time.Unix(int64(rs.Cloudbrain.CreatedUnix), 0).Format(CREATE_TIME_FORMAT), getCellName("F", row): getBrainWaitTime(rs),
		getCellName("G", row): rs.TrainJobDuration, getCellName("H", row): getBrainStartTime(rs),
		getCellName("I", row): getBrainEndTime(rs),
		getCellName("J", row): rs.ComputeResource, getCellName("K", row): rs.Name, getCellName("L", row): getBrainRepo(rs),
		getCellName("M", row): rs.JobName, getCellName("N", row): getBrainDeleteTime(rs),
	}
}
func getBrainRepo(rs *models.CloudbrainInfo) string {
	if rs.Repo != nil {
		return rs.Repo.OwnerName + "/" + rs.Repo.Alias
	}
	return ""
}
func getBrainStartTime(rs *models.CloudbrainInfo) string {
	timeString := time.Unix(int64(rs.Cloudbrain.StartTime), 0).Format(CREATE_TIME_FORMAT)
	if timeString != "1970/01/01 08:00:00" {
		return timeString
	} else {
		return "0"
	}

}
func getBrainEndTime(rs *models.CloudbrainInfo) string {
	timeString := time.Unix(int64(rs.Cloudbrain.EndTime), 0).Format(CREATE_TIME_FORMAT)
	if timeString != "1970/01/01 08:00:00" {
		return timeString
	} else {
		return "0"
	}

}
func getBrainWaitTime(rs *models.CloudbrainInfo) string {
	var waitTime int64
	if rs.Cloudbrain.Status == string(models.JobWaiting) {
		waitTime = time.Now().Unix() - rs.Cloudbrain.CreatedUnix.AsTime().Unix()
	} else {
		waitTime = int64(rs.Cloudbrain.StartTime - rs.Cloudbrain.CreatedUnix)
	}
	if waitTime <= 0 {
		return "00:00:00"
	} else {
		return models.ConvertDurationToStr(waitTime)
	}
}
func getCloudbrainType(rs *models.CloudbrainInfo, ctx *context.Context) string {
	if rs.Cloudbrain.Type == models.TypeCloudBrainOne {
		return ctx.Tr("repo.cloudbrain1")
	} else if rs.Cloudbrain.Type == models.TypeCloudBrainTwo {
		return ctx.Tr("repo.cloudbrain2")
	} else if rs.Cloudbrain.Type == models.TypeC2Net {
		return ctx.Tr("repo.intelligent_net")
	} else {
		return ctx.Tr("repo.cloudbrain_untype")
	}
}

func getBrainDeleteTime(rs *models.CloudbrainInfo) string {
	nilTime := time.Time{}
	if rs.Cloudbrain.DeletedAt != nilTime {
		return rs.Cloudbrain.DeletedAt.Format("2006-01-02 15:04:05")
	} else {
		return ""
	}
}
func getCloudbrainTimePeroid(ctx *context.Context, recordBeginTime time.Time) (time.Time, time.Time, error) {
	queryType := ctx.QueryTrim("type")
	beginTimeStr := ctx.QueryTrim("beginTime")
	endTimeStr := ctx.QueryTrim("endTime")
	now := time.Now()

	var beginTime time.Time
	var endTime time.Time
	var err error
	if queryType != "" {

		if queryType == "all" {
			beginTime = recordBeginTime
			endTime = now
		} else if queryType == "today" {
			endTime = now
			beginTime = time.Date(endTime.Year(), endTime.Month(), endTime.Day(), 0, 0, 0, 0, now.Location())
		} else if queryType == "yesterday" {
			endTime = time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location())
			beginTime = endTime.AddDate(0, 0, -1)

		} else if queryType == "last_7day" {
			beginTime = now.AddDate(0, 0, -6)
			beginTime = time.Date(beginTime.Year(), beginTime.Month(), beginTime.Day(), 0, 0, 0, 0, now.Location())
			endTime = now
		} else if queryType == "last_30day" {
			beginTime = now.AddDate(0, 0, -29)
			beginTime = time.Date(beginTime.Year(), beginTime.Month(), beginTime.Day(), 0, 0, 0, 0, now.Location())
			endTime = now
		} else if queryType == "current_month" {
			endTime = now
			beginTime = time.Date(endTime.Year(), endTime.Month(), 1, 0, 0, 0, 0, now.Location())
		} else if queryType == "monthly" {
			endTime = now
			beginTime = now.AddDate(0, -1, 1)
			beginTime = time.Date(beginTime.Year(), beginTime.Month(), beginTime.Day(), 0, 0, 0, 0, now.Location())

		} else if queryType == "current_year" {
			endTime = now
			beginTime = time.Date(endTime.Year(), 1, 1, 0, 0, 0, 0, now.Location())

		} else if queryType == "last_month" {

			lastMonthTime := now.AddDate(0, -1, 0)
			beginTime = time.Date(lastMonthTime.Year(), lastMonthTime.Month(), 1, 0, 0, 0, 0, now.Location())
			endTime = time.Date(now.Year(), now.Month(), 1, 0, 0, 0, 0, now.Location())

		} else {
			return now, now, fmt.Errorf("The value of type parameter is wrong.")

		}

	} else {
		if beginTimeStr == "" || endTimeStr == "" {
			//如果查询类型和开始时间结束时间都未设置，按queryType=all处理
			beginTime = recordBeginTime
			endTime = now

		} else {

			beginTime, err = time.ParseInLocation("2006-01-02", beginTimeStr, time.Local)
			if err != nil {
				return now, now, err
			}

			endTime, err = time.ParseInLocation("2006-01-02", endTimeStr, time.Local)
			if err != nil {
				return now, now, err
			}
		}

	}

	if beginTime.Before(recordBeginTime) {
		beginTime = recordBeginTime
	}

	return beginTime, endTime, nil
}
